@@ -245,6 +245,9 @@ warn  "Redundant list comprehension" = [x | x <- y] ==> y where _ = isVar x
 
 error "Redundant seq" = x `seq` x ==> x
 error "Redundant $!" = id $! x ==> x
+error "Redundant seq" = x `seq` y ==> y where _ = isWHNF x
+error "Redundant $!" = f $! x ==> f x where _ = isWHNF x
+error "Redundant evaluate" = evaluate x ==> return x where _ = isWHNF x
 
 -- MAYBE
 
@@ -465,6 +468,10 @@ import Prelude hiding (catch); no = catch
 import Control.Exception as E; no = E.catch
 main = do f; putStrLn $ show x -- print x
 main = map (writer,) $ map arcObj $ filter (rdfPredEq (Res dctreferences)) ts -- map ((writer,) . arcObj) (filter (rdfPredEq (Res dctreferences)) ts)
+h x y = return $! (x, y) -- return (x, y)
+h x y = return $! x
+getInt = do { x <- readIO "0"; return $! (x :: Int) }
+foo = evaluate [12] -- return [12]
 
 import Prelude \
 yes = flip mapM -- Control.Monad.forM
