@@ -3,6 +3,7 @@
 module CmdLine(Cmd(..), CppFlags(..), getCmd, exitWithHelp) where
 
 import Control.Monad
+import Data.Char
 import Data.List
 import Data.Maybe
 import System.Console.GetOpt
@@ -39,7 +40,7 @@ data Cmd = Cmd
     ,cmdFindHints :: [FilePath]      -- ^ source files to look for hints in
     ,cmdLanguage :: [Extension]      -- ^ the extensions (may be prefixed by "No")
     ,cmdQuiet :: Bool                -- ^ supress all console output
-    ,cmdCross :: Bool                -- ^ work between source files
+    ,cmdCross :: Bool                -- ^ work between source files, applies to hints such as duplicate code between modules
     }
 
 
@@ -47,6 +48,7 @@ data Opts = Help
           | Ver
           | Test
           | Hints FilePath
+          | Path FilePath
           | Report FilePath
           | Skip String
           | ShowAll
@@ -80,11 +82,12 @@ opts = [Option "?" ["help"] (NoArg Help) "Display help message"
        ,Option "f" ["find"] (ReqArg FindHints "file") "Find hints in a Haskell file"
        ,Option "t" ["test"] (NoArg Test) "Run in test mode"
        ,Option "d" ["datadir"] (ReqArg DataDir "dir") "Override the data directory"
+       ,Option "p" ["path"] (ReqArg Path "dir") "Directory in which to search for files"
        ,Option "q" ["quiet"] (NoArg Quiet) "Supress most console output"
        ,Option ""  ["cpp-define"] (ReqArg Define "name[=value]") "CPP #define"
        ,Option ""  ["cpp-include"] (ReqArg Include "dir") "CPP include path"
        ,Option ""  ["cpp-simple"] (NoArg SimpleCpp) "Use a simple CPP (strip # lines)"
-       ,Option ""  ["cpp-ansi"] (NoArg Ansi) "Use CPP in -ansi mode"
+       ,Option ""  ["cpp-ansi"] (NoArg Ansi) "Use CPP in ANSI compatibility mode"
        ]
 
 
@@ -107,8 +110,9 @@ getCmd args = do
 
     let exts = [x | Ext x <- opt]
         exts2 = if null exts then ["hs","lhs"] else exts
-    files <- if null files then return Nothing else fmap Just $ concatMapM (getFile exts2) files
-    findHints <- concatMapM (getFile exts2) [x | FindHints x <- opt]
+    let path = [x | Path x <- opt] ++ ["."]
+    files <- if null files then return Nothing else fmap Just $ concatMapM (getFile path exts2) files
+    findHints <- concatMapM (getFile path exts2) [x | FindHints x <- opt]
 
     let hintFiles = [x | Hints x <- opt]
     hints <- mapM (getHintFile dataDir) $ hintFiles ++ ["HLint" | null hintFiles]
@@ -167,20 +171,46 @@ helpText = unlines
     ,"  hlint src --report"
     ]
 
+"." <\> x = x
+x <\> y = x </> y
 
-getFile :: [String] -> FilePath -> IO [FilePath]
-getFile exts file = do
-    b <- doesDirectoryExist file
-    if b then do
-        xs <- getDirectoryContentsRecursive file
+
+getFile :: [FilePath] -> [String] -> FilePath -> IO [FilePath]
+getFile path _ "-" = return ["-"]
+getFile [] exts file = error $ "Couldn't find file: " ++ file
+getFile (p:ath) exts file = do
+    isDir <- doesDirectoryExist $ p <\> file
+    if isDir then do
+        xs <- getDirectoryContentsRecursive $ p <\> file
         return [x | x <- xs, drop 1 (takeExtension x) `elem` exts]
      else do
-        b <- doesFileExist file
-        unless b $ error $ "Couldn't find file: " ++ file
-        return [file]
+        isFil <- doesFileExist $ p <\> file
+        if isFil then return [p <\> file]
+         else do
+            res <- getModule p exts file
+            case res of
+                Just x -> return [x]
+                Nothing -> getFile ath exts file
+
+
+getModule :: FilePath -> [String] -> FilePath -> IO (Maybe FilePath)
+getModule path exts x | not (any isSpace x) && all isMod xs = f exts
+    where
+        xs = words $ map (\x -> if x == '.' then ' ' else x) x
+        isMod (x:xs) = isUpper x && all (\x -> isAlphaNum x || x == '_') xs
+        isMod _ = False
+        pre = path <\> joinPath xs
+
+        f [] = return Nothing
+        f (x:xs) = do
+            let s = pre <.> x
+            b <- doesFileExist s
+            if b then return $ Just s else f xs
+getModule _ _ _ = return Nothing
 
 
 getHintFile :: FilePath -> FilePath -> IO FilePath
+getHintFile _ "-" = return "-"
 getHintFile dataDir x = do
         let poss = nub $ concat [x : [x <.> "hs" | takeExtension x /= ".hs"] | x <- [x,dataDir </> x]]
         f poss poss
