@@ -23,19 +23,22 @@ import Hint.Duplicate
 
 staticHints :: [(String,Hint)]
 staticHints =
-    let x*y = (x,DeclHint y) ; x+y = (x,ModuHint y) in
-    ["List"       * listHint
-    ,"ListRec"    * listRecHint
-    ,"Monad"      * monadHint
-    ,"Lambda"     * lambdaHint
-    ,"Bracket"    * bracketHint
-    ,"Naming"     * namingHint
-    ,"Structure"  * structureHint
+    ["List"       ! listHint
+    ,"ListRec"    ! listRecHint
+    ,"Monad"      ! monadHint
+    ,"Lambda"     ! lambdaHint
+    ,"Bracket"    ! bracketHint
+    ,"Naming"     ! namingHint
+    ,"Structure"  ! structureHint
     ,"Import"     + importHint
     ,"Pragma"     + pragmaHint
     ,"Extensions" + extensionsHint
-    ,"Duplicate"  + duplicateHint
+    ,"Duplicate"  * duplicateHint
     ]
+    where
+        x!y = (x,DeclHint y)
+        x+y = (x,ModuHint y)
+        x*y = (x,CrossHint y)
 
 dynamicHints :: [Setting] -> Hint
 dynamicHints = DeclHint . readMatch
