@@ -55,6 +55,9 @@ f = \x -> bar map (filter x) where -- bar map . filter
 f = bar &+& \x -> f (g x)
 foo = [\column -> set column [treeViewColumnTitle := printf "%s (match %d)" name (length candidnates)]]
 foo = [\x -> x]
+foo = [\m x -> insert x x m]
+foo a b c = bar (flux ++ quux) c where flux = a -- foo a b = bar (flux ++ quux)
+foo a b c = bar (flux ++ quux) c where flux = c
 </TEST>
 -}
 
@@ -64,6 +67,7 @@ module Hint.Lambda where
 import Hint.Util
 import Hint.Type
 import Util
+import Data.Maybe
 
 
 lambdaHint :: DeclHint
@@ -71,10 +75,11 @@ lambdaHint _ _ x = concatMap (uncurry lambdaExp) (universeParentBi x) ++ concatM
 
 
 lambdaDecl :: Decl_ -> [Idea]
-lambdaDecl (toFunBind -> o@(FunBind _ [Match _ name pats (UnGuardedRhs _ bod) Nothing]))
-    | isLambda $ fromParen bod = [err "Redundant lambda" o $ uncurry reform $ fromLambda $ Lambda an pats bod]
-    | (pats2,bod2) <- etaReduce pats bod, length pats2 < length pats = [err "Eta reduce" o $ reform pats2 bod2]
-        where reform p b = FunBind an [Match an name p (UnGuardedRhs an b) Nothing]
+lambdaDecl (toFunBind -> o@(FunBind loc [Match _ name pats (UnGuardedRhs _ bod) bind]))
+    | isNothing bind, isLambda $ fromParen bod = [err "Redundant lambda" o $ uncurry reform $ fromLambda $ Lambda an pats bod]
+    | (pats2,bod2) <- etaReduce pats bod, length pats2 < length pats, pvars (drop (length pats2) pats) `disjoint` vars bind
+        = [err "Eta reduce" (reform pats bod) (reform pats2 bod2)]
+        where reform p b = FunBind loc [Match an name p (UnGuardedRhs an b) Nothing]
 lambdaDecl _ = []
 
 
