@@ -3,7 +3,7 @@
     Suggest the use of camelCase
 
     Only permit:
-    _*[A-Za-z]*_?'?
+    _*[A-Za-z]*_*'*
 
     Apply this to things that would get exported by default only
     Also allow prop_ as it's a standard QuickCheck idiom
@@ -19,6 +19,8 @@ yes_foo = yes_foo + yes_foo -- yesFoo = ...
 no = 1 where yes_foo = 2
 a -== b = 1
 myTest = 1; my_test = 1
+semiring'laws = 1 -- semiringLaws = ...
+data Yes = FOO_A | Foo_B -- data Yes = FOO_A | FooB
 </TEST>
 -}
 
@@ -73,16 +75,16 @@ getNames x = case x of
 
 
 suggestName :: String -> Maybe String
-suggestName x = listToMaybe [f x | not $ isSym x || good || "prop_" `isPrefixOf` x]
+suggestName x = listToMaybe [f x | not $ isSym x || good || not (any isLower x) || "prop_" `isPrefixOf` x]
     where
-        good = all isAlphaNum $ drp '_' $ drp '\'' $ reverse $ dropWhile (== '_') x
-        drp x ys = if [x] `isPrefixOf` ys then tail ys else ys
+        good = all isAlphaNum $ drp '_' $ drp '\'' $ reverse $ drp '_' x
+        drp x = dropWhile (== x)
 
         f xs = us ++ g ys
             where (us,ys) = span (== '_') xs
 
         g x | x `elem` ["_","'","_'"] = x
-        g ('_':x:xs) | isAlphaNum x = toUpper x : g xs
+        g (a:x:xs) | a `elem` "_'" && isAlphaNum x = toUpper x : g xs
         g (x:xs) | isAlphaNum x = x : g xs
                  | otherwise = g xs
         g [] = []
